@@ -19,10 +19,11 @@
 import glob
 import os
 import re
-import six
+import time
 
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
+import six
 
 from os_brick import exception
 from os_brick import executor
@@ -31,8 +32,8 @@ from os_brick import utils
 
 LOG = logging.getLogger(__name__)
 
-MULTIPATH_ERROR_REGEX = re.compile("\w{3} \d+ \d\d:\d\d:\d\d \|.*$")
-MULTIPATH_WWID_REGEX = re.compile("\((?P<wwid>.+)\)")
+MULTIPATH_ERROR_REGEX = re.compile(r"\w{3} \d+ \d\d:\d\d:\d\d \|.*$")
+MULTIPATH_WWID_REGEX = re.compile(r"\((?P<wwid>.+)\)")
 MULTIPATH_DEVICE_ACTIONS = ['unchanged:', 'reject:', 'reload:',
                             'switchpg:', 'rename:', 'create:',
                             'resize:']
@@ -60,32 +61,44 @@ class LinuxSCSI(executor.Executor):
         else:
             return None
 
-    def remove_scsi_device(self, device, force=False, exc=None):
+    def remove_scsi_device(self, device, force=False, exc=None,
+                           flush=True):
         """Removes a scsi device based upon /dev/sdX name."""
         path = "/sys/block/%s/device/delete" % device.replace("/dev/", "")
         if os.path.exists(path):
             exc = exception.ExceptionChainer() if exc is None else exc
-            # flush any outstanding IO first
-            with exc.context(force, 'Flushing %s failed', device):
-                self.flush_device_io(device)
+            if flush:
+                # flush any outstanding IO first
+                with exc.context(force, 'Flushing %s failed', device):
+                    self.flush_device_io(device)
 
             LOG.debug("Remove SCSI device %(device)s with %(path)s",
                       {'device': device, 'path': path})
             with exc.context(force, 'Removing %s failed', device):
                 self.echo_scsi_command(path, "1")
 
-    @utils.retry(exceptions=exception.VolumePathNotRemoved)
     def wait_for_volumes_removal(self, volumes_names):
         """Wait for device paths to be removed from the system."""
         str_names = ', '.join(volumes_names)
         LOG.debug('Checking to see if SCSI volumes %s have been removed.',
                   str_names)
-        exist = [volume_name for volume_name in volumes_names
-                 if os.path.exists('/dev/' + volume_name)]
-        if exist:
-            LOG.debug('%s still exist.', ', '.join(exist))
-            raise exception.VolumePathNotRemoved(volume_path=exist)
-        LOG.debug("SCSI volumes %s have been removed.", str_names)
+        exist = ['/dev/' + volume_name for volume_name in volumes_names]
+
+        # It can take up to 30 seconds to remove a SCSI device if the path
+        # failed right before we start detaching, which is unlikely, but we
+        # still shouldn't fail in that case.
+        for i in range(61):
+            exist = [path for path in exist if os.path.exists(path)]
+            if not exist:
+                LOG.debug("SCSI volumes %s have been removed.", str_names)
+                return
+            # Don't sleep on the last try since we are quitting
+            if i < 60:
+                time.sleep(0.5)
+                # Log every 5 seconds
+                if i % 10 == 0:
+                    LOG.debug('%s still exist.', ', '.join(exist))
+        raise exception.VolumePathNotRemoved(volume_path=exist)
 
     def get_device_info(self, device):
         (out, _err) = self._execute('sg_scan', device, run_as_root=True,
@@ -155,8 +168,13 @@ class LinuxSCSI(executor.Executor):
         try:
             if execute is None:
                 execute = priv_rootwrap.execute
-            execute('multipathd', 'show', 'status',
-                    run_as_root=True, root_helper=root_helper)
+            cmd = ('multipathd', 'show', 'status')
+            out, _err = execute(*cmd, run_as_root=True,
+                                root_helper=root_helper)
+            # There was a bug in multipathd where it didn't return an error
+            # code and just printed the error message in stdout.
+            if out and out.startswith('error receiving packet'):
+                raise putils.ProcessExecutionError('', out, 1, cmd, None)
         except putils.ProcessExecutionError as err:
             LOG.error('multipathd is not running: exit code %(err)s',
                       {'err': err.exit_code})
@@ -193,14 +211,48 @@ class LinuxSCSI(executor.Executor):
                 return dm
         return None
 
+    @staticmethod
+    def get_dev_path(connection_properties, device_info):
+        """Determine what path was used by Nova/Cinder to access volume."""
+        if device_info and device_info.get('path'):
+            return device_info.get('path')
+
+        return connection_properties.get('device_path') or ''
+
+    @staticmethod
+    def requires_flush(path, path_used, was_multipath):
+        """Check if a device needs to be flushed when detaching.
+
+        A device representing a single path connection to a volume must only be
+        flushed if it has been used directly by Nova or Cinder to write data.
+
+        If the path has been used via a multipath DM or if the device was part
+        of a multipath but a different single path was used for I/O (instead of
+        the multipath) then we don't need to flush.
+        """
+        # No used path happens on failed attachs, when we don't care about
+        # individual flushes.
+        # When Nova used a multipath we don't need to do individual flushes.
+        if not path_used or was_multipath:
+            return False
+
+        # We need to flush the single path that was used.
+        # For encrypted volumes the symlink has been replaced, so realpath
+        # won't return device under /dev but under /dev/disk/...
+        path = os.path.realpath(path)
+        path_used = os.path.realpath(path_used)
+        return path_used == path or '/dev' != os.path.split(path_used)[0]
+
     def remove_connection(self, devices_names, is_multipath, force=False,
-                          exc=None):
+                          exc=None, path_used=None, was_multipath=False):
         """Remove LUNs and multipath associated with devices names.
 
         :param devices_names: Iterable with real device names ('sda', 'sdb')
         :param is_multipath: Whether this is a multipath connection or not
         :param force: Whether to forcefully disconnect even if flush fails.
         :param exc: ExceptionChainer where to add exceptions if forcing
+        :param path_used: What path was used by Nova/Cinder for I/O
+        :param was_multipath: If the path used for I/O was a multipath
         :returns: Multipath device map name if found and not flushed
         """
         if not devices_names:
@@ -220,7 +272,9 @@ class LinuxSCSI(executor.Executor):
                     multipath_name = None
 
         for device_name in devices_names:
-            self.remove_scsi_device('/dev/' + device_name, force, exc)
+            dev_path = '/dev/' + device_name
+            flush = self.requires_flush(dev_path, path_used, was_multipath)
+            self.remove_scsi_device(dev_path, force, exc, flush)
 
         # Wait until the symlinks are removed
         with exc.context(force, 'Some devices remain from %s', devices_names):
@@ -391,7 +445,8 @@ class LinuxSCSI(executor.Executor):
             lines = out.strip()
             lines = lines.split("\n")
             lines = [line for line in lines
-                     if not re.match(MULTIPATH_ERROR_REGEX, line)]
+                     if not re.match(MULTIPATH_ERROR_REGEX, line) and
+                     len(line)]
             if lines:
 
                 mdev_name = lines[0].split(" ")[0]
@@ -571,7 +626,7 @@ class LinuxSCSI(executor.Executor):
         """Find the device name given a session and the hctl.
 
         :param session: A string with the session number
-        "param hctl: An iterable with the host, channel, target, and lun as
+        :param hctl: An iterable with the host, channel, target, and lun as
                      passed to scan.  ie: ('5', '-', '-', '0')
         """
         if '-' in hctl:
