@@ -259,9 +259,7 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
                                          devices['devices'][0])
         expected_commands = [
             'multipath -f ' + find_mp_device_path_mock.return_value,
-            'blockdev --flushbufs /dev/sdb',
             'tee -a /sys/block/sdb/device/delete',
-            'blockdev --flushbufs /dev/sdc',
             'tee -a /sys/block/sdc/device/delete',
         ]
         self.assertEqual(expected_commands, self.cmds)
@@ -593,7 +591,29 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
                                      ('1234567890123457', 2)]
             }
         },
-
+        {
+            "target_info": {
+                "target_lun": 1,
+                "target_wwn": ['20320002AC01E166', '21420002AC01E166',
+                               '20410002AC01E166', '21410002AC01E166']
+            },
+            "expected_targets": [
+                ('20320002ac01e166', 1),
+                ('21420002ac01e166', 1),
+                ('20410002ac01e166', 1),
+                ('21410002ac01e166', 1)
+            ],
+            "itmap": {
+                '10001409DCD71FF6': ['20320002AC01E166', '21420002AC01E166'],
+                '10001409DCD71FF7': ['20410002AC01E166', '21410002AC01E166']
+            },
+            "expected_map": {
+                '10001409dcd71ff6': [('20320002ac01e166', 1),
+                                     ('21420002ac01e166', 1)],
+                '10001409dcd71ff7': [('20410002ac01e166', 1),
+                                     ('21410002ac01e166', 1)]
+            }
+        },
     )
     @ddt.unpack
     def test__add_targets_to_connection_properties(self, target_info,
@@ -605,15 +625,42 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
         conn = self.fibrechan_connection(volume, "10.0.2.15:3260", wwn)
         conn['data'].update(target_info)
 
-        if itmap:
-            conn['data']['initiator_target_map'] = itmap
+        conn['data']['initiator_target_map'] = itmap
 
         connection_info = self.connector._add_targets_to_connection_properties(
             conn['data'])
         self.assertIn('targets', connection_info)
         self.assertEqual(expected_targets, connection_info['targets'])
 
+        # Check that we turn to lowercase target wwns
+        key = 'target_wwns' if 'target_wwns' in target_info else 'target_wwn'
+        wwns = target_info.get(key)
+        wwns = [wwns] if isinstance(wwns, six.string_types) else wwns
+        wwns = [w.lower() for w in wwns]
+        if wwns:
+            self.assertEqual(wwns, conn['data'][key])
+
         if itmap:
             self.assertIn('initiator_target_lun_map', connection_info)
             self.assertEqual(expected_map,
                              connection_info['initiator_target_lun_map'])
+
+    @ddt.data(('/dev/mapper/<WWN>', True),
+              ('/dev/mapper/mpath0', True),
+              ('/dev/disk/by-path/pci-1-fc-1-lun-1', False))
+    @ddt.unpack
+    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.remove_scsi_device')
+    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.requires_flush')
+    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.get_dev_path')
+    def test__remove_devices(self, path_used, was_multipath, get_dev_path_mock,
+                             flush_mock, remove_mock):
+        get_dev_path_mock.return_value = path_used
+        self.connector._remove_devices(mock.sentinel.con_props,
+                                       [{'device': '/dev/sda'}],
+                                       mock.sentinel.device_info)
+        get_dev_path_mock.assert_called_once_with(mock.sentinel.con_props,
+                                                  mock.sentinel.device_info)
+        flush_mock.assert_called_once_with('/dev/sda', path_used,
+                                           was_multipath)
+        remove_mock.assert_called_once_with('/dev/sda',
+                                            flush=flush_mock.return_value)
