@@ -17,12 +17,14 @@ import sys
 
 import mock
 from oslo_concurrency import processutils as putils
+from oslo_service import loopingcall
 
 from os_brick import exception
 from os_brick.initiator import connector
 from os_brick.initiator.connectors import base
 from os_brick.initiator.connectors import fake
 from os_brick.initiator.connectors import iscsi
+from os_brick.initiator.connectors import nvme
 from os_brick.initiator import linuxfc
 from os_brick.privileged import rootwrap as priv_rootwrap
 from os_brick.tests import base as test_base
@@ -31,8 +33,16 @@ MY_IP = '10.0.0.1'
 FAKE_SCSI_WWN = '1234567890'
 
 
+class ZeroIntervalLoopingCall(loopingcall.FixedIntervalLoopingCall):
+    def start(self, interval, initial_delay=None, stop_on_exception=True):
+        return super(ZeroIntervalLoopingCall, self).start(
+            0, 0, stop_on_exception)
+
+
 class ConnectorUtilsTestCase(test_base.TestCase):
 
+    @mock.patch.object(nvme.NVMeConnector, '_get_system_uuid',
+                       return_value=None)
     @mock.patch.object(iscsi.ISCSIConnector, 'get_initiator',
                        return_value='fakeinitiator')
     @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_wwpns',
@@ -46,6 +56,7 @@ class ConnectorUtilsTestCase(test_base.TestCase):
                                              multipath_result,
                                              mock_wwnns, mock_wwpns,
                                              mock_initiator,
+                                             mock_sysuuid,
                                              host='fakehost'):
         props_actual = connector.get_connector_properties('sudo',
                                                           MY_IP,
@@ -89,7 +100,7 @@ class ConnectorUtilsTestCase(test_base.TestCase):
     def test_brick_get_connector_properties(self):
         self._test_brick_get_connector_properties(False, False, False)
 
-    @mock.patch.object(priv_rootwrap, 'execute')
+    @mock.patch.object(priv_rootwrap, 'execute', return_value=('', ''))
     def test_brick_get_connector_properties_multipath(self, mock_execute):
         self._test_brick_get_connector_properties(True, True, True)
         mock_execute.assert_called_once_with('multipathd', 'show', 'status',
@@ -122,6 +133,8 @@ class ConnectorTestCase(test_base.TestCase):
     def setUp(self):
         super(ConnectorTestCase, self).setUp()
         self.cmds = []
+        self.mock_object(loopingcall, 'FixedIntervalLoopingCall',
+                         ZeroIntervalLoopingCall)
 
     def fake_execute(self, *cmd, **kwargs):
         self.cmds.append(" ".join(cmd))
@@ -149,7 +162,7 @@ class ConnectorTestCase(test_base.TestCase):
 
     def test_get_connector_properties(self):
         with mock.patch.object(priv_rootwrap, 'execute') as mock_exec:
-            mock_exec.return_value = True
+            mock_exec.return_value = ('', '')
             multipath = True
             enforce_multipath = True
             props = base.BaseLinuxConnector.get_connector_properties(
@@ -178,6 +191,25 @@ class ConnectorTestCase(test_base.TestCase):
                 'sudo', multipath=multipath,
                 enforce_multipath=enforce_multipath)
 
+    @mock.patch('sys.platform', 'win32')
+    def test_get_connector_mapping_win32(self):
+        mapping_win32 = connector.get_connector_mapping()
+        self.assertTrue('ISCSI' in mapping_win32)
+        self.assertFalse('RBD' in mapping_win32)
+        self.assertFalse('STORPOOL' in mapping_win32)
+
+    @mock.patch('os_brick.initiator.connector.platform.machine')
+    def test_get_connector_mapping(self, mock_platform_machine):
+        mock_platform_machine.return_value = 'x86_64'
+        mapping_x86 = connector.get_connector_mapping()
+        mock_platform_machine.return_value = 'ppc64le'
+        mapping_ppc = connector.get_connector_mapping()
+        self.assertNotEqual(mapping_x86, mapping_ppc)
+        mock_platform_machine.return_value = 's390x'
+        mapping_s390 = connector.get_connector_mapping()
+        self.assertNotEqual(mapping_x86, mapping_s390)
+        self.assertNotEqual(mapping_ppc, mapping_s390)
+
     def test_factory(self):
         obj = connector.InitiatorConnector.factory('iscsi', None)
         self.assertEqual("ISCSIConnector", obj.__class__.__name__)
