@@ -19,6 +19,7 @@ import time
 
 import ddt
 import mock
+from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
 
 from os_brick import exception
@@ -33,6 +34,7 @@ class LinuxSCSITestCase(base.TestCase):
     def setUp(self):
         super(LinuxSCSITestCase, self).setUp()
         self.cmds = []
+        self.realpath = os.path.realpath
         self.mock_object(os.path, 'realpath', return_value='/dev/sdc')
         self.mock_object(os, 'stat', returns=os.stat(__file__))
         self.linuxscsi = linuxscsi.LinuxSCSI(None, execute=self.fake_execute)
@@ -91,10 +93,20 @@ class LinuxSCSITestCase(base.TestCase):
         flush_mock.assert_called_once_with(device)
         echo_mock.assert_called_once_with('/sys/block/sdc/device/delete', '1')
 
+    @mock.patch.object(os.path, 'exists', return_value=False)
+    def test_remove_scsi_device_no_flush(self, exists_mock):
+        self.linuxscsi.remove_scsi_device("/dev/sdc")
+        expected_commands = []
+        self.assertEqual(expected_commands, self.cmds)
+        exists_mock.return_value = True
+        self.linuxscsi.remove_scsi_device("/dev/sdc", flush=False)
+        expected_commands = [('tee -a /sys/block/sdc/device/delete')]
+        self.assertEqual(expected_commands, self.cmds)
+
     @mock.patch('time.sleep')
     @mock.patch('os.path.exists', return_value=True)
     def test_wait_for_volumes_removal_failure(self, exists_mock, sleep_mock):
-        retries = 3
+        retries = 61
         names = ('sda', 'sdb')
         self.assertRaises(exception.VolumePathNotRemoved,
                           self.linuxscsi.wait_for_volumes_removal, names)
@@ -209,10 +221,10 @@ class LinuxSCSITestCase(base.TestCase):
     @mock.patch.object(time, 'sleep')
     def test_wait_for_path_not_found(self, exists_mock, sleep_mock):
         path = "/dev/disk/by-id/dm-uuid-mpath-%s" % '1234567890'
-        self.assertRaisesRegexp(exception.VolumeDeviceNotFound,
-                                r'Volume device not found at %s' % path,
-                                self.linuxscsi.wait_for_path,
-                                path)
+        self.assertRaisesRegex(exception.VolumeDeviceNotFound,
+                               r'Volume device not found at %s' % path,
+                               self.linuxscsi.wait_for_path,
+                               path)
 
     @ddt.data({'do_raise': False, 'force': False},
               {'do_raise': True, 'force': True})
@@ -243,8 +255,8 @@ class LinuxSCSITestCase(base.TestCase):
         self.assertEqual(get_dm_name_mock.return_value if do_raise else None,
                          mp_name)
         remove_mock.assert_has_calls([
-            mock.call('/dev/sda', mock.sentinel.Force, exc),
-            mock.call('/dev/sdb', mock.sentinel.Force, exc)])
+            mock.call('/dev/sda', mock.sentinel.Force, exc, False),
+            mock.call('/dev/sdb', mock.sentinel.Force, exc, False)])
         wait_mock.assert_called_once_with(devices_names)
         self.assertEqual(do_raise, bool(exc))
         remove_link_mock.assert_called_once_with(devices_names)
@@ -285,8 +297,28 @@ class LinuxSCSITestCase(base.TestCase):
                                          force=mock.sentinel.Force,
                                          exc=exc)
         remove_mock.assert_has_calls(
-            [mock.call('/dev/sda', mock.sentinel.Force, exc),
-             mock.call('/dev/sdb', mock.sentinel.Force, exc)])
+            [mock.call('/dev/sda', mock.sentinel.Force, exc, False),
+             mock.call('/dev/sdb', mock.sentinel.Force, exc, False)])
+        wait_mock.assert_called_once_with(devices_names)
+        remove_link_mock.assert_called_once_with(devices_names)
+
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
+    def test_remove_connection_singlepath_used(self, remove_mock, wait_mock,
+                                               remove_link_mock):
+        devices_names = ('sda', 'sdb')
+        exc = exception.ExceptionChainer()
+
+        # realpath was mocked on test setup
+        with mock.patch('os.path.realpath', side_effect=self.realpath):
+            self.linuxscsi.remove_connection(devices_names, is_multipath=True,
+                                             force=mock.sentinel.Force,
+                                             exc=exc, path_used='/dev/sdb',
+                                             was_multipath=False)
+        remove_mock.assert_has_calls(
+            [mock.call('/dev/sda', mock.sentinel.Force, exc, False),
+             mock.call('/dev/sdb', mock.sentinel.Force, exc, True)])
         wait_mock.assert_called_once_with(devices_names)
         remove_link_mock.assert_called_once_with(devices_names)
 
@@ -420,6 +452,44 @@ class LinuxSCSITestCase(base.TestCase):
         self.assertEqual("0", info['devices'][1]['id'])
         self.assertEqual("3", info['devices'][1]['lun'])
 
+    def test_find_multipath_device_with_multiple_errors(self):
+        def fake_execute(*cmd, **kwargs):
+            out = ("Jun 21 04:39:26 | 8:160: path wwid appears to have "
+                   "changed. Using old wwid.\n\n"
+                   "Jun 21 04:39:26 | 65:208: path wwid appears to have "
+                   "changed. Using old wwid.\n\n"
+                   "Jun 21 04:39:26 | 65:208: path wwid appears to have "
+                   "changed. Using old wwid.\n"
+                   "3624a93707edcfde1127040370004ee62 dm-84 PURE    ,"
+                   "FlashArray\n"
+                   "size=100G features='0' hwhandler='0' wp=rw\n"
+                   "`-+- policy='queue-length 0' prio=1 status=active\n"
+                   "  |- 8:0:0:9  sdaa 65:160 active ready running\n"
+                   "  `- 8:0:1:9  sdac 65:192 active ready running\n"
+                   )
+            return out, None
+
+        self.linuxscsi._execute = fake_execute
+
+        info = self.linuxscsi.find_multipath_device('/dev/sdaa')
+
+        self.assertEqual("3624a93707edcfde1127040370004ee62", info["id"])
+        self.assertEqual("3624a93707edcfde1127040370004ee62", info["name"])
+        self.assertEqual("/dev/mapper/3624a93707edcfde1127040370004ee62",
+                         info["device"])
+
+        self.assertEqual("/dev/sdaa", info['devices'][0]['device'])
+        self.assertEqual("8", info['devices'][0]['host'])
+        self.assertEqual("0", info['devices'][0]['channel'])
+        self.assertEqual("0", info['devices'][0]['id'])
+        self.assertEqual("9", info['devices'][0]['lun'])
+
+        self.assertEqual("/dev/sdac", info['devices'][1]['device'])
+        self.assertEqual("8", info['devices'][1]['host'])
+        self.assertEqual("0", info['devices'][1]['channel'])
+        self.assertEqual("1", info['devices'][1]['id'])
+        self.assertEqual("9", info['devices'][1]['lun'])
+
     @mock.patch.object(time, 'sleep')
     def test_wait_for_rw(self, mock_sleep):
         lsblk_output = """3624a93709a738ed78583fd1200143029 (dm-2)  0
@@ -760,12 +830,20 @@ loop0                                     0"""
         expected = 13
         self.assertEqual(expected, result)
 
-    @mock.patch('os_brick.privileged.rootwrap')
-    def test_is_multipath_running_default_executor(self, mock_rootwrap):
-        self.assertTrue(
-            linuxscsi.LinuxSCSI.is_multipath_running(
-                False, None, mock_rootwrap.execute))
-        mock_rootwrap.execute.assert_called_once_with(
+    @mock.patch('os_brick.privileged.rootwrap.execute', return_value=('', ''))
+    def test_is_multipath_running_default_executor(self, mock_exec):
+        res = linuxscsi.LinuxSCSI.is_multipath_running(False, None, mock_exec)
+        self.assertTrue(res)
+        mock_exec.assert_called_once_with(
+            'multipathd', 'show', 'status', run_as_root=True, root_helper=None)
+
+    @mock.patch('os_brick.privileged.rootwrap.execute')
+    def test_is_multipath_running_failure_exit_code_0(self, mock_exec):
+        mock_exec.return_value = ('error receiving packet', '')
+        self.assertRaises(putils.ProcessExecutionError,
+                          linuxscsi.LinuxSCSI.is_multipath_running,
+                          True, None, mock_exec)
+        mock_exec.assert_called_once_with(
             'multipathd', 'show', 'status', run_as_root=True, root_helper=None)
 
     @mock.patch('glob.glob')
@@ -953,3 +1031,50 @@ loop0                                     0"""
     def test_multipath_add_path(self):
         self.linuxscsi.multipath_add_path('/dev/sda')
         self.assertEqual(['multipathd add path /dev/sda'], self.cmds)
+
+    @ddt.data({'con_props': {}, 'dev_info': {'path': mock.sentinel.path}},
+              {'con_props': None, 'dev_info': {'path': mock.sentinel.path}},
+              {'con_props': {'device_path': mock.sentinel.device_path},
+               'dev_info': {'path': mock.sentinel.path}})
+    @ddt.unpack
+    def test_get_dev_path_device_info(self, con_props, dev_info):
+        self.assertEqual(mock.sentinel.path,
+                         self.linuxscsi.get_dev_path(con_props, dev_info))
+
+    @ddt.data({'con_props': {'device_path': mock.sentinel.device_path},
+               'dev_info': {'path': None}},
+              {'con_props': {'device_path': mock.sentinel.device_path},
+               'dev_info': {'path': ''}},
+              {'con_props': {'device_path': mock.sentinel.device_path},
+               'dev_info': {}},
+              {'con_props': {'device_path': mock.sentinel.device_path},
+               'dev_info': None})
+    @ddt.unpack
+    def test_get_dev_path_conn_props(self, con_props, dev_info):
+        self.assertEqual(mock.sentinel.device_path,
+                         self.linuxscsi.get_dev_path(con_props, dev_info))
+
+    @ddt.data({'con_props': {'device_path': ''}, 'dev_info': {'path': None}},
+              {'con_props': {'device_path': None}, 'dev_info': {'path': ''}},
+              {'con_props': {}, 'dev_info': {}},
+              {'con_props': {}, 'dev_info': None})
+    @ddt.unpack
+    def test_get_dev_path_no_path(self, con_props, dev_info):
+        self.assertEqual('', self.linuxscsi.get_dev_path(con_props, dev_info))
+
+    @ddt.data(('/dev/sda', '/dev/sda', True, False, None),
+              ('/dev/sda', '', True, False, None),
+              ('/dev/link_sda', '/dev/link_sdb', False, False, ('/dev/sda',
+                                                                '/dev/sdb')),
+              ('/dev/link_sda', '/dev/link2_sda', False, True, ('/dev/sda',
+                                                                '/dev/sda')))
+    @ddt.unpack
+    def test_requires_flush(self, path, path_used, was_multipath, expected,
+                            real_paths):
+        with mock.patch('os.path.realpath', side_effect=real_paths) as mocked:
+            self.assertEqual(
+                expected,
+                self.linuxscsi.requires_flush(path, path_used, was_multipath))
+            if real_paths:
+                mocked.assert_has_calls([mock.call(path),
+                                         mock.call(path_used)])
